use { gtk4 :: { ListBoxRow , Align :: * } , libadwaita :: * , std :: cell :: * } ;

use crate :: {
	utility :: * ,
	ui :: { * , utility :: * , collatable_container :: SortingDirection :: * } ,
} ;



pub struct FilmSortButton {
	widget : SplitButton ,
	previous_sorting : & 'static RefCell <FilmsSorting> ,
}
pub struct SeriesSortButton {
	widget : SplitButton ,
	previous_sorting : & 'static RefCell <SeriesSorting> ,
}

impl FilmSortButton {
	pub fn new < F : Fn (FilmsSorting) + 'static > ( on_sort : F ) -> Self {
		let previous_sorting = leak ( RefCell :: new ( FilmsSorting :: default () ) ) ;

		let widget = split_button ! (
			@ popover : popover ! (
				@ css_classes : [ "menu" ] ,
				list_box ! (
					@ connect_row_activated : move | _ , row | {
						on_film_sort_activated ( row , previous_sorting , & on_sort ) ;
					} ,
					g_box ! (
						@ orientation : Horizontal , @ spacing : 20 ,
						label ! ( @ hexpand : true , @ halign : Start , "Name" ) ,
						icon ! ("view-sort-ascending-symbolic") ,
					) ,
					g_box ! (
						@ orientation : Horizontal , @ spacing : 20 ,
						label ! ( @ hexpand : true , @ halign : Start , "Release date" ) ,
						icon ! ("view-sort-ascending-symbolic") ,
					) ,
					g_box ! (
						@ orientation : Horizontal , @ spacing : 20 ,
						label ! ( @ hexpand : true , @ halign : Start , "Runtime" ) ,
						icon ! ("view-sort-ascending-symbolic") ,
					) ,
				) ,
			) ,
			label ! ("Sort") ,
		) ;

		Self { widget , previous_sorting }
	}
}
impl SeriesSortButton {
	pub fn new < F : Fn (SeriesSorting) + 'static > ( on_sort : F ) -> Self {
		let previous_sorting = leak ( RefCell :: new ( SeriesSorting :: default () ) ) ;

		let widget = split_button ! (
			@ popover : popover ! (
				@ css_classes : [ "menu" ] ,
				list_box ! (
					@ connect_row_activated : move | _ , row | {
						on_series_sort_activated ( row , previous_sorting , & on_sort ) ;
					} ,
					g_box ! (
						@ orientation : Horizontal , @ spacing : 20 ,
						label ! ( @ hexpand : true , @ halign : Start , "Name" ) ,
						icon ! ("view-sort-ascending-symbolic") ,
					) ,
					g_box ! (
						@ orientation : Horizontal , @ spacing : 20 ,
						label ! ( @ hexpand : true , @ halign : Start , "First release date" ) ,
						icon ! ("view-sort-ascending-symbolic") ,
					) ,
				) ,
			) ,
			label ! ("Sort") ,
		) ;

		Self { widget , previous_sorting }
	}
}

impl Component <SplitButton> for FilmSortButton {
	fn get_widget ( & self ) -> & SplitButton { & self . widget }
}
impl Component <SplitButton> for SeriesSortButton {
	fn get_widget ( & self ) -> & SplitButton { & self . widget }
}

fn on_film_sort_activated < F : Fn (FilmsSorting) > (
	row : & ListBoxRow ,
	previous_sorting : & RefCell <FilmsSorting> ,
	on_sort : & F ,
) {
	let sorting_property = match row . index () {
		0 => FilmProperty :: Name ,
		1 => FilmProperty :: ReleaseDate ,
		2 => FilmProperty :: Runtime ,
		_ => panic ! () ,
	} ;

	if sorting_property == previous_sorting . borrow () . property {
		let previous_sorting_direction = previous_sorting . borrow () . direction ;
		match previous_sorting_direction {
			Ascending => {
				previous_sorting . replace ( FilmsSorting :: new ( sorting_property , Descending ) ) ;
				on_sort ( FilmsSorting :: new ( sorting_property , Descending ) ) ;
			} ,
			Descending => {
				previous_sorting . replace ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
				on_sort ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
			} ,
		}
	} else {
		previous_sorting . replace ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
		on_sort ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
	}
}
fn on_series_sort_activated < F : Fn (SeriesSorting) > (
	row : & ListBoxRow ,
	previous_sorting : & RefCell <SeriesSorting> ,
	on_sort : & F ,
) {
	let sorting_property = match row . index () {
		0 => SeriesProperty :: Name ,
		1 => SeriesProperty :: FirstReleaseDate ,
		_ => panic ! () ,
	} ;

	if sorting_property == previous_sorting . borrow () . property {
		let previous_sorting_direction = previous_sorting . borrow () . direction ;
		match previous_sorting_direction {
			Ascending => {
				previous_sorting . replace ( SeriesSorting :: new ( sorting_property , Descending ) ) ;
				on_sort ( SeriesSorting :: new ( sorting_property , Descending ) ) ;
			} ,
			Descending => {
				previous_sorting . replace ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
				on_sort ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
			} ,
		}
	} else {
		previous_sorting . replace ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
		on_sort ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
	}
}
